@@ -92,6 +92,7 @@ class ActivityEntry(gtk.Entry):
         self.connect("focus-out-event", self._on_focus_out_event)
         self.connect("changed", self._on_text_changed)
         self.connect("parent-set", self._on_parent_set)
+        self._parent_click_watcher = None # bit lame but works
 
         runtime.dispatcher.add_handler('activity_updated', self.after_activity_update)
 
@@ -99,6 +100,9 @@ class ActivityEntry(gtk.Entry):
         self.populate_suggestions()
 
     def hide_popup(self):
+        if self._parent_click_watcher and self.get_toplevel().handler_is_connected(self._parent_click_watcher):
+            self.get_toplevel().disconnect(self._parent_click_watcher)
+            self._parent_click_watcher = None
         self.popup.hide()
 
     def show_popup(self):
@@ -107,6 +111,9 @@ class ActivityEntry(gtk.Entry):
             self.hide_popup()
             return
 
+        if not self._parent_click_watcher:
+            self._parent_click_watcher = self.get_toplevel().connect("button-press-event", self._on_focus_out_event)
+
         activity = stuff.parse_activity_input(self.filter)
         time = ''
         if activity.start_time:
