@@ -19,6 +19,7 @@
 
 from .hamster.stuff import format_duration
 import gtk
+from gtk import keysyms
 import datetime as dt
 import calendar
 import gobject
@@ -60,6 +61,8 @@ class TimeInput(gtk.Entry):
         self.connect("key-press-event", self._on_key_press_event)
         self.connect("focus-in-event", self._on_focus_in_event)
         self.connect("focus-out-event", self._on_focus_out_event)
+        self._parent_click_watcher = None # bit lame but works
+
         self.connect("changed", self._on_text_changed)
         self.show()
 
@@ -119,7 +122,7 @@ class TimeInput(gtk.Entry):
 
         self.set_text(time_text)
         self.set_position(len(time_text))
-        self.popup.hide()
+        self.hide_popup()
         if self.news:
             self.emit("time-entered")
             self.news = False
@@ -142,15 +145,22 @@ class TimeInput(gtk.Entry):
         self.show_popup()
 
     def _on_focus_out_event(self, event, something):
-        self.popup.hide()
+        self.hide_popup()
         if self.news:
             self.emit("time-entered")
             self.news = False
 
+    def hide_popup(self):
+        if self._parent_click_watcher and self.get_toplevel().handler_is_connected(self._parent_click_watcher):
+            self.get_toplevel().disconnect(self._parent_click_watcher)
+            self._parent_click_watcher = None
+        self.popup.hide()
 
     def show_popup(self):
-        # will be going either 24 hours or from start time to start time + 12 hours
+        if not self._parent_click_watcher:
+            self._parent_click_watcher = self.get_toplevel().connect("button-press-event", self._on_focus_out_event)
 
+        # will be going either 24 hours or from start time to start time + 12 hours
         start_time = dt.datetime.combine(dt.date.today(), self.start_time) # we will be adding things
         i_time = start_time # we will be adding things
 
@@ -218,13 +228,17 @@ class TimeInput(gtk.Entry):
 
 
     def _on_key_press_event(self, entry, event):
-        cursor = self.time_tree.get_cursor()
+        if event.keyval not in (keysyms.Up, keysyms.Down, keysyms.Return, keysyms.KP_Enter):
+            #any kind of other input
+            self.hide_popup()
+            return False
 
-        if not cursor or not cursor[0]:
+        model, iter = self.time_tree.get_selection().get_selected()
+        if not iter:
             return
 
-        i = cursor[0][0]
 
+        i = model.get_path(iter)[0]
         if event.keyval == gtk.keysyms.Up:
             i-=1
         elif event.keyval == gtk.keysyms.Down:
@@ -237,15 +251,18 @@ class TimeInput(gtk.Entry):
             else:
                 self._select_time(entry.get_text())
         elif (event.keyval == gtk.keysyms.Escape):
-            self.popup.hide()
-        else:
-            #any kind of other input
-            self.popup.hide()
-            return False
+            self.hide_popup()
+            return
 
-        # keep it in the sane borders
+        # keep it in sane limits
         i = min(max(i, 0), len(self.time_tree.get_model()) - 1)
 
         self.time_tree.set_cursor(i)
         self.time_tree.scroll_to_cell(i, use_align = True, row_align = 0.4)
+
+        # if popup is not visible, display it on up and down
+        if event.keyval in (gtk.keysyms.Up, gtk.keysyms.Down) and self.popup.props.visible == False:
+            self.show_popup()
+
         return True
+
